@@ -78,7 +78,7 @@ rasqal_new_0op_expression(rasqal_world* world, rasqal_op op)
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
 
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -126,7 +126,7 @@ rasqal_new_1op_expression(rasqal_world* world, rasqal_op op,
       goto tidy;
   }
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -175,7 +175,7 @@ rasqal_new_2op_expression(rasqal_world* world,
   if(!world || !arg1 || !arg2)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -222,7 +222,7 @@ rasqal_new_3op_expression(rasqal_world* world,
   if(!world || !arg1 || !arg2) /* arg3 may be NULL */
     goto tidy;
 
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -271,7 +271,7 @@ rasqal_new_string_op_expression(rasqal_world* world,
   if(!world || !arg1 || !literal)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -308,7 +308,7 @@ rasqal_new_literal_expression(rasqal_world* world, rasqal_literal *literal)
   if(!world || !literal)
     return NULL;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {  
     e->usage = 1;
     e->world = world;
@@ -335,7 +335,7 @@ rasqal_new_function_expression_common(rasqal_world* world,
   if(!world || (arg1 && args) || (name && !args)|| (!name && args))
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -437,7 +437,7 @@ rasqal_new_cast_expression(rasqal_world* world, raptor_uri* name,
   if(!world || !name || !value)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -478,7 +478,7 @@ rasqal_new_expr_seq_expression(rasqal_world* world,
   if(!world || !args)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -517,7 +517,7 @@ rasqal_new_set_expression(rasqal_world* world, rasqal_op op,
   if(!world || !arg1 || !args)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -562,7 +562,7 @@ rasqal_new_group_concat_expression(rasqal_world* world,
   if(!world || !args)
     goto tidy;
   
-  e = (rasqal_expression*)RASQAL_CALLOC(rasqal_expression, 1, sizeof(*e));
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
   if(e) {
     e->usage = 1;
     e->world = world;
@@ -680,6 +680,12 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       if(e->arg1)
         rasqal_free_expression(e->arg1);
@@ -885,6 +891,12 @@ rasqal_expression_visit(rasqal_expression* e,
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       return (e->arg1) ? rasqal_expression_visit(e->arg1, fn, user_data) : 0;
       break;
@@ -1024,7 +1036,13 @@ static const char* const rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "abs",
   "round",
   "ceil",
-  "floor"
+  "floor",
+  "md5",
+  "sha1",
+  "sha224",
+  "sha256",
+  "sha384",
+  "sha512"
 };
 
 
@@ -1203,6 +1221,12 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
       raptor_iostream_write_byte('(', iostr);
@@ -1415,6 +1439,12 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
@@ -1615,6 +1645,12 @@ rasqal_expression_is_constant(rasqal_expression* e)
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       /* arg1 is optional for RASQAL_EXPR_BNODE and result is always constant */
       result = (e->arg1) ? rasqal_expression_is_constant(e->arg1) : 1;
       break;
@@ -1962,6 +1998,12 @@ rasqal_expression_compare(rasqal_expression* e1, rasqal_expression* e2,
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       /* arg1 is optional for RASQAL_EXPR_BNODE */
       rc = rasqal_expression_compare(e1->arg1, e2->arg1, flags, error_p);
       break;
@@ -2145,8 +2187,7 @@ rasqal_expression_convert_aggregate_to_variable(rasqal_expression* e_in,
   world = e_in->world;
   
   if(e_out) {
-    *e_out = (rasqal_expression*)RASQAL_MALLOC(rasqal_expression,
-                                               sizeof(**e_out));
+    *e_out = RASQAL_MALLOC(rasqal_expression*, sizeof(**e_out));
     if(!*e_out)
       goto tidy;
   }
@@ -2201,7 +2242,7 @@ rasqal_new_evaluation_context(rasqal_world* world,
   
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
 
-  eval_context = (rasqal_evaluation_context*)RASQAL_CALLOC(rasqal_evaluation_context, 1, sizeof(*eval_context));
+  eval_context = RASQAL_CALLOC(rasqal_evaluation_context*, 1, sizeof(*eval_context));
   if(!eval_context)
     return NULL;
   
