@@ -44,6 +44,9 @@
 
 #define DEBUG_FH stderr
 
+/* Enable evaluation debugging by defining RASQAL_DEBUG_EVAL */
+/* #define RASQAL_DEBUG_EVAL 1 */
+
 
 /* 
  * rasqal_language_matches:
@@ -172,7 +175,7 @@ rasqal_expression_evaluate_strdt(rasqal_expression *e,
       goto failed;
   }
   
-  new_s =(unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  new_s =RASQAL_MALLOC(unsigned char*, len + 1);
   if(!new_s)
     goto failed;
   memcpy(new_s, s, len + 1);
@@ -190,7 +193,7 @@ rasqal_expression_evaluate_strdt(rasqal_expression *e,
     *error_p = 1;
   
   if(new_s)
-    RASQAL_FREE(cstring, new_s);
+    RASQAL_FREE(char*, new_s);
   if(l1)
     rasqal_free_literal(l1);
   if(l2)
@@ -247,12 +250,12 @@ rasqal_expression_evaluate_strlang(rasqal_expression *e,
   if(*error_p)
     goto failed;
   
-  new_s = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  new_s = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!new_s)
     goto failed;
   memcpy(new_s, s, len + 1);
   
-  new_lang = (unsigned char*)RASQAL_MALLOC(cstring, lang_len + 1);
+  new_lang = RASQAL_MALLOC(unsigned char*, lang_len + 1);
   if(!new_lang)
     goto failed;
   memcpy(new_lang, lang, lang_len + 1);
@@ -270,7 +273,7 @@ rasqal_expression_evaluate_strlang(rasqal_expression *e,
     *error_p = 1;
   
   if(new_s)
-    RASQAL_FREE(cstring, new_s);
+    RASQAL_FREE(char*, new_s);
   if(l1)
     rasqal_free_literal(l1);
   if(l2)
@@ -454,7 +457,7 @@ rasqal_expression_evaluate_sameterm(rasqal_expression *e,
     goto failed;
 
   b = rasqal_literal_same_term(l1, l2);
-#if RASQAL_DEBUG > 1
+#ifdef RASQAL_DEBUG_EVAL
   RASQAL_DEBUG2("rasqal_literal_same_term() returned: %d\n", b);
 #endif
 
@@ -510,7 +513,7 @@ rasqal_expression_evaluate_in_set(rasqal_expression *e,
     
     found = (rasqal_literal_equals_flags(l1, arg_literal, 
                                          eval_context->flags, error_p) != 0);
-#if RASQAL_DEBUG > 1
+#ifdef RASQAL_DEBUG_EVAL
     if(*error_p)
       RASQAL_DEBUG1("rasqal_literal_equals_flags() returned: FAILURE\n");
     else
@@ -610,7 +613,7 @@ rasqal_expression_evaluate_str(rasqal_expression *e,
   if(!s || *error_p)
     goto failed;
   
-  new_s = (unsigned char *)RASQAL_MALLOC(cstring, len + 1);
+  new_s = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!new_s)
     goto failed;
 
@@ -675,13 +678,13 @@ rasqal_expression_evaluate_lang(rasqal_expression *e,
   
   if(l1->language) {
     size_t len = strlen(l1->language);
-    new_s = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    new_s = RASQAL_MALLOC(unsigned char*, len + 1);
     if(!new_s)
       goto failed;
 
     memcpy((char*)new_s, l1->language, len + 1);
   } else  {
-    new_s = (unsigned char*)RASQAL_MALLOC(cstring, 1);
+    new_s = RASQAL_MALLOC(unsigned char*, 1);
     if(!new_s)
       goto failed;
 
@@ -855,7 +858,7 @@ rasqal_expression_evaluate_bnode_constructor(rasqal_expression *e,
     if(*error_p)
       goto failed;
 
-    new_s = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    new_s = RASQAL_MALLOC(unsigned char*, len + 1);
     if(!new_s)
       goto failed;
 
@@ -934,7 +937,7 @@ rasqal_expression_evaluate2(rasqal_expression* e,
 
   errs.e = 0;
 
-#ifdef RASQAL_DEBUG
+#ifdef RASQAL_DEBUG_EVAL
   RASQAL_DEBUG2("evaluating expression %p: ", e);
   rasqal_expression_print(e, stderr);
   fprintf(stderr, "\n");
@@ -1027,12 +1030,14 @@ rasqal_expression_evaluate2(rasqal_expression* e,
        */
       if(!rasqal_xsd_datatype_check(l1->type, l1->string, flags) ||
          !rasqal_xsd_datatype_check(l2->type, l2->string, flags)) {
+#ifdef RASQAL_DEBUG_EVAL
         RASQAL_DEBUG1("One of the literals was invalid\n");
+#endif
         goto failed;
       }
 
       vars.b = (rasqal_literal_equals_flags(l1, l2, flags, &errs.e) != 0);
-#if RASQAL_DEBUG > 1
+#ifdef RASQAL_DEBUG_EVAL
       if(errs.e)
         RASQAL_DEBUG1("rasqal_literal_equals_flags returned: FAILURE\n");
       else
@@ -1058,7 +1063,7 @@ rasqal_expression_evaluate2(rasqal_expression* e,
       }
 
       vars.b = (rasqal_literal_not_equals_flags(l1, l2, flags, &errs.e) != 0);
-#if RASQAL_DEBUG > 1
+#ifdef RASQAL_DEBUG_EVAL
       if(errs.e)
         RASQAL_DEBUG1("rasqal_literal_not_equals_flags returned: FAILURE\n");
       else
@@ -1375,10 +1380,9 @@ rasqal_expression_evaluate2(rasqal_expression* e,
       break;
 
     case RASQAL_EXPR_FUNCTION:
-      rasqal_log_error_simple(world,
-                              RAPTOR_LOG_LEVEL_WARN,
-                              eval_context->locator,
-                              "No function expressions support at present.  Returning false.");
+      rasqal_log_warning_simple(world, RASQAL_WARNING_LEVEL_NOT_IMPLEMENTED,
+                                eval_context->locator,
+                                "No function expressions support at present.  Returning false.");
       result = rasqal_new_boolean_literal(world, 0);
       break;
       
@@ -1534,6 +1538,15 @@ rasqal_expression_evaluate2(rasqal_expression* e,
       result = rasqal_expression_evaluate_floor(e, eval_context, error_p);
       break;
 
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
+      result = rasqal_expression_evaluate_digest(e, eval_context, error_p);
+      break;
+
     case RASQAL_EXPR_UNKNOWN:
     default:
       RASQAL_FATAL3("Unknown operation %s (%d)",
@@ -1542,7 +1555,7 @@ rasqal_expression_evaluate2(rasqal_expression* e,
 
   got_result:
 
-#ifdef RASQAL_DEBUG
+#ifdef RASQAL_DEBUG_EVAL
   RASQAL_DEBUG2("result of %p: ", e);
   rasqal_expression_print(e, stderr);
   fputs( ": ", stderr);
