@@ -53,7 +53,9 @@
 #define DEBUG_FH stderr
 
 
-static size_t
+#if RAPTOR_VERSION < 20005
+
+static int
 rasqal_unicode_utf8_strlen(const unsigned char *string, size_t length)
 {
   size_t unicode_length = 0;
@@ -61,8 +63,10 @@ rasqal_unicode_utf8_strlen(const unsigned char *string, size_t length)
   while(length > 0) {
     int unichar_len;
     unichar_len = raptor_unicode_utf8_string_get_char(string, length, NULL);
-    if(unichar_len < 0 || unichar_len > (int)length)
+    if(unichar_len < 0 || unichar_len > (int)length) {
+      unicode_length = -1;
       break;
+    }
     
     string += unichar_len;
     length -= unichar_len;
@@ -70,7 +74,7 @@ rasqal_unicode_utf8_strlen(const unsigned char *string, size_t length)
     unicode_length++;
   }
 
-  return unicode_length;
+  return (int)unicode_length;
 }
 
 
@@ -88,11 +92,11 @@ rasqal_unicode_utf8_strlen(const unsigned char *string, size_t length)
  * If @dest is NULL, returns the number of bytes needed to write and
  * does no work.
  * 
- * Return value: number of bytes used in destination string
+ * Return value: number of bytes used in destination string or 0 on failure
  */
 static size_t
 rasqal_unicode_utf8_substr(unsigned char* dest, size_t* dest_length_p,
-                           const unsigned char* src, int src_length,
+                           const unsigned char* src, size_t src_length,
                            int startingLoc, int length)
 {
   size_t dest_length = 0; /* destination unicode characters count */
@@ -138,6 +142,11 @@ rasqal_unicode_utf8_substr(unsigned char* dest, size_t* dest_length_p,
   return dest_bytes;
 }
 
+#define raptor_unicode_utf8_strlen rasqal_unicode_utf8_strlen
+#define raptor_unicode_utf8_substr rasqal_unicode_utf8_substr
+
+#endif
+
 
 /* 
  * rasqal_expression_evaluate_strlen:
@@ -157,7 +166,7 @@ rasqal_expression_evaluate_strlen(rasqal_expression *e,
   rasqal_literal* l1;
   rasqal_literal* result = NULL;
   const unsigned char *s;
-  size_t len = 0;
+  int len = 0;
   
   l1 = rasqal_expression_evaluate2(e->arg1, eval_context, error_p);
   if(*error_p || !l1)
@@ -170,10 +179,11 @@ rasqal_expression_evaluate_strlen(rasqal_expression *e,
   if(!s)
     len = 0;
   else
-    len = rasqal_unicode_utf8_strlen(s, strlen((const char*)s));
+    len = raptor_unicode_utf8_strlen(s, strlen((const char*)s));
   
 
-  result = rasqal_new_integer_literal(world, RASQAL_LITERAL_INTEGER, len);
+  result = rasqal_new_numeric_literal_from_long(world, RASQAL_LITERAL_INTEGER,
+                                                len);
   rasqal_free_literal(l1);
   return result;
 
@@ -244,17 +254,18 @@ rasqal_expression_evaluate_substr(rasqal_expression *e,
 
   }
   
-  new_s = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  new_s = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!new_s)
     goto failed;
 
   /* adjust starting index to xsd fn:substring initial offset 1 */
-  rasqal_unicode_utf8_substr(new_s, /* dest_length_p */ NULL,
-                             s, len, startingLoc - 1, length);
+  if(!raptor_unicode_utf8_substr(new_s, /* dest_length_p */ NULL,
+                                 s, len, startingLoc - 1, length))
+    goto failed;
 
   if(l1->language) {
     len = strlen((const char*)l1->language);
-    new_lang = (char*)RASQAL_MALLOC(cstring, len + 1);
+    new_lang = RASQAL_MALLOC(char*, len + 1);
     if(!new_lang)
       goto failed;
 
@@ -322,7 +333,7 @@ rasqal_expression_evaluate_set_case(rasqal_expression *e,
   if(*error_p)
     goto failed;
 
-  new_s =(unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  new_s =RASQAL_MALLOC(unsigned char*, len + 1);
   if(!new_s)
     goto failed;
 
@@ -349,7 +360,7 @@ rasqal_expression_evaluate_set_case(rasqal_expression *e,
 
   if(l1->language) {
     len = strlen((const char*)l1->language);
-    new_lang = (char*)RASQAL_MALLOC(cstring, len + 1);
+    new_lang = RASQAL_MALLOC(char*, len + 1);
     if(!new_lang)
       goto failed;
 
@@ -372,9 +383,9 @@ rasqal_expression_evaluate_set_case(rasqal_expression *e,
     *error_p = 1;
   
   if(new_s)
-    RASQAL_FREE(cstring, new_s);
+    RASQAL_FREE(char*, new_s);
   if(new_lang)
-    RASQAL_FREE(cstring, new_lang);
+    RASQAL_FREE(char*, new_lang);
   if(l1)
     rasqal_free_literal(l1);
 
@@ -564,7 +575,7 @@ rasqal_expression_evaluate_encode_for_uri(rasqal_expression *e,
     goto failed;
 
   /* pessimistically assume every UTF-8 byte is %XX 3 x len */
-  new_s = (unsigned char*)RASQAL_MALLOC(cstring, (3 * len) + 1);
+  new_s = RASQAL_MALLOC(unsigned char*, (3 * len) + 1);
   if(!new_s)
     goto failed;
 
@@ -606,7 +617,7 @@ rasqal_expression_evaluate_encode_for_uri(rasqal_expression *e,
     *error_p = 1;
   
   if(new_s)
-    RASQAL_FREE(cstring, new_s);
+    RASQAL_FREE(char*, new_s);
   if(l1)
     rasqal_free_literal(l1);
 
@@ -692,7 +703,7 @@ rasqal_expression_evaluate_concat(rasqal_expression *e,
   
   
   len = raptor_stringbuffer_length(sb);
-  result_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  result_str = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!result_str)
     goto failed;
   
@@ -712,7 +723,7 @@ rasqal_expression_evaluate_concat(rasqal_expression *e,
   if(dt)
     raptor_free_uri(dt);
   if(result_str)
-    RASQAL_FREE(cstring, result_str);
+    RASQAL_FREE(char*, result_str);
   if(sb)
     raptor_free_stringbuffer(sb);
   if(result)
