@@ -45,7 +45,7 @@
 
 
 static int rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter, raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
-static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter, rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri);
+static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter, rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri, unsigned int flags);
 
 
 #if RASQAL_DEBUG > 1
@@ -496,6 +496,8 @@ typedef struct
   /* Variables table allocated for variables in the result set */
   rasqal_variables_table* vars_table;
   int variables_count;
+
+  unsigned int flags;
 } rasqal_rowsource_sparql_xml_context;
   
 
@@ -585,14 +587,17 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
         unsigned char* var_name;
         rasqal_variable *v;
         
-        var_name = (unsigned char*)RASQAL_MALLOC(cstring, con->name_length+1);
+        var_name = RASQAL_MALLOC(unsigned char*, con->name_length + 1);
         memcpy(var_name, con->name, con->name_length + 1);
 
         v = rasqal_variables_table_add(con->vars_table,
                                        RASQAL_VARIABLE_TYPE_NORMAL,
                                        var_name, NULL);
-        if(v)
+        if(v) {
           rasqal_rowsource_add_variable(con->rowsource, v);
+          /* above function takes a reference to v */
+          rasqal_free_variable(v);
+        }
       }
       break;
       
@@ -644,7 +649,7 @@ rasqal_sparql_xml_sax2_characters_handler(void *user_data,
      con->state == STATE_uri ||
      con->state == STATE_bnode) {
     con->value_len=len;
-    con->value=(char*)RASQAL_MALLOC(cstring, len+1);
+    con->value = RASQAL_MALLOC(char*, len + 1);
     memcpy(con->value, s, len);
     con->value[len]='\0';
   }
@@ -700,13 +705,13 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         raptor_uri* datatype_uri=NULL;
         char* language_str=NULL;
 
-        lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
+        lvalue = RASQAL_MALLOC(unsigned char*, con->value_len + 1);
         memcpy(lvalue, con->value, con->value_len + 1);
         if(con->datatype)
           datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
         if(con->language) {
           size_t language_len = strlen(con->language);
-          language_str=(char*)RASQAL_MALLOC(cstring, language_len + 1);
+          language_str = RASQAL_MALLOC(char*, language_len + 1);
           memcpy(language_str, con->language, language_len + 1);
         }
         l = rasqal_new_string_literal_node(con->world, lvalue, language_str,
@@ -722,7 +727,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         rasqal_literal* l;
         unsigned char* lvalue;
-        lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
+        lvalue = RASQAL_MALLOC(unsigned char*, con->value_len + 1);
         memcpy(lvalue, con->value, con->value_len + 1);
         l = rasqal_new_simple_literal(con->world, RASQAL_LITERAL_BLANK, lvalue);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
@@ -763,7 +768,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
   }
 
   if(con->value) {
-    RASQAL_FREE(cstring, con->value);
+    RASQAL_FREE(char*, con->value);
     con->value=NULL;
   }
 }
@@ -810,6 +815,11 @@ rasqal_rowsource_sparql_xml_finish(rasqal_rowsource* rowsource, void *user_data)
   if(con->vars_table)
     rasqal_free_variables_table(con->vars_table);
 
+  if(con->flags) {
+    if(con->iostr)
+      raptor_free_iostream(con->iostr);
+  }
+
   RASQAL_FREE(rasqal_rowsource_sparql_xml_context, con);
 
   return 0;
@@ -832,7 +842,6 @@ rasqal_rowsource_sparql_xml_process(rasqal_rowsource_sparql_xml_context* con)
     if(read_len > 0) {
       RASQAL_DEBUG2("processing %d bytes\n", (int)read_len);
       raptor_sax2_parse_chunk(con->sax2, con->buffer, read_len, 0);
-      con->locator.byte += read_len;
     }
     
     if(read_len < FILE_READ_BUF_SIZE) {
@@ -893,7 +902,7 @@ static const rasqal_rowsource_handler rasqal_rowsource_sparql_xml_handler={
   /* .read_row = */ rasqal_rowsource_sparql_xml_read_row,
   /* .read_all_rows = */ NULL,
   /* .reset = */ NULL,
-  /* .set_preserve = */ NULL,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ NULL,
 };
@@ -916,11 +925,12 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* fo
                                               rasqal_world *world,
                                               rasqal_variables_table* vars_table,
                                               raptor_iostream *iostr,
-                                              raptor_uri *base_uri)
+                                              raptor_uri *base_uri,
+                                              unsigned int flags)
 {
   rasqal_rowsource_sparql_xml_context* con;
   
-  con=(rasqal_rowsource_sparql_xml_context*)RASQAL_CALLOC(rasqal_rowsource_sparql_xml_context, 1, sizeof(rasqal_rowsource_sparql_xml_context));
+  con = RASQAL_CALLOC(rasqal_rowsource_sparql_xml_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
@@ -933,6 +943,8 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* fo
   con->sax2 = raptor_new_sax2(world->raptor_world_ptr, &con->locator, con);
   if(!con->sax2)
     return NULL;
+
+  con->flags = flags;
   
   raptor_sax2_set_start_element_handler(con->sax2,
                                         rasqal_sparql_xml_sax2_start_element_handler);
