@@ -63,7 +63,11 @@ const char * const rasqal_home_url_string = "http://librdf.org/rasqal/";
  *
  * See also #rasqal_version_decimal.
  */
-const char * const rasqal_version_string = RASQAL_VERSION_STRING;
+const char * const rasqal_version_string = RASQAL_VERSION_STRING
+#ifdef GIT_VERSION
+" GIT " GIT_VERSION
+#endif
+;
 
 /**
  * rasqal_version_major:
@@ -110,7 +114,14 @@ const unsigned int rasqal_version_decimal = RASQAL_VERSION;
 rasqal_world*
 rasqal_new_world(void)
 {
-  return (rasqal_world*)RASQAL_CALLOC(rasqal_world, sizeof(rasqal_world), 1);
+  rasqal_world* world;
+
+  world = RASQAL_CALLOC(rasqal_world*, sizeof(rasqal_world), 1);
+  if(!world)
+    return NULL;
+  
+  world->warning_level = RASQAL_WARNING_LEVEL_DEFAULT;
+  return world;
 }
 
 
@@ -189,16 +200,9 @@ rasqal_world_open(rasqal_world *world)
 #endif
 
 
-#ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
   rc = rasqal_raptor_init(world);
   if(rc)
     return rc;
-#endif
-#ifdef RAPTOR_TRIPLES_SOURCE_REDLAND
-  rc = rasqal_redland_init(world);
-  if(rc)
-    return rc;
-#endif
 
   rc = rasqal_init_query_results();
   if(rc)
@@ -293,6 +297,9 @@ rasqal_world_get_raptor(rasqal_world* world)
  * @handler: log handler function
  *
  * Set the log handler for this rasqal_world.
+ *
+ * Also sets the raptor log handler to the same @user_data and
+ * @handler via raptor_world_set_log_handler(). (Rasqal 0.9.26+)
  **/
 void
 rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
@@ -302,6 +309,8 @@ rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
 
   world->log_handler = handler;
   world->log_handler_user_data = user_data;
+
+  raptor_world_set_log_handler(world->raptor_world_ptr, user_data, handler);
 }
 
 
@@ -353,8 +362,7 @@ rasqal_query_language_register_factory(rasqal_world *world,
 {
   rasqal_query_language_factory *query = NULL;
   
-  query = (rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
-                                                        sizeof(*query));
+  query = RASQAL_CALLOC(rasqal_query_language_factory*, 1, sizeof(*query));
   if(!query)
     goto tidy;
 
@@ -452,7 +460,7 @@ rasqal_world_get_query_language_description(rasqal_world* world,
 
   rasqal_world_open(world);
 
-  factory = raptor_sequence_get_at(world->query_languages, counter);
+  factory = (rasqal_query_language_factory *)raptor_sequence_get_at(world->query_languages, counter);
   if(!factory)
     return NULL;
 
@@ -489,7 +497,7 @@ rasqal_languages_enumerate(rasqal_world *world,
   /* for compatibility with old API that does not call this - FIXME Remove V2 */
   rasqal_world_open(world);
   
-  factory = raptor_sequence_get_at(world->query_languages, counter);
+  factory = (rasqal_query_language_factory*)raptor_sequence_get_at(world->query_languages, counter);
   if(!factory)
     return 1;
 
@@ -552,6 +560,24 @@ rasqal_log_error_simple(rasqal_world* world, raptor_log_level level,
 }
 
 
+/* internal */
+void
+rasqal_log_warning_simple(rasqal_world* world,
+                          rasqal_warning_level warn_level,
+                          raptor_locator* locator, const char* message, ...)
+{
+  va_list arguments;
+
+  if(warn_level > world->warning_level)
+    return;
+
+  va_start(arguments, message);
+  rasqal_log_error_varargs(world, RAPTOR_LOG_LEVEL_WARN,
+                           locator, message, arguments);
+  va_end(arguments);
+}
+
+
 void
 rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
                          raptor_locator* locator,
@@ -606,7 +632,7 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
     fputc('\n', stderr);
   }
 
-  RASQAL_FREE(cstring, buffer);
+  RASQAL_FREE(char*, buffer);
 }
 
 
@@ -701,7 +727,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
   unsigned char *endp;
   int n;
   
-  result=(unsigned char*)RASQAL_MALLOC(cstring, len+1);
+  result = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!result)
     return NULL;
 
@@ -719,7 +745,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
         if(error_handler)
           error_handler(error_data, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
         /* UTF-8 encoding had an error or ended in the middle of a string */
-        RASQAL_FREE(cstring, result);
+        RASQAL_FREE(char*, result);
         return NULL;
       }
       memcpy(dest, p, unichar_len);
@@ -738,7 +764,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
     }
 
     if(!len) {
-      RASQAL_FREE(cstring, result);
+      RASQAL_FREE(char*, result);
       return NULL;
     }
 
@@ -756,7 +782,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
         if(len < ulen) {
           if(error_handler)
             error_handler(error_data, "%c over end of line", c);
-          RASQAL_FREE(cstring, result);
+          RASQAL_FREE(char*, result);
           return 0;
         }
         
@@ -782,7 +808,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
       default:
         if(error_handler)
           error_handler(error_data, "Illegal string escape \\%c in \"%s\"", c, src);
-        RASQAL_FREE(cstring, result);
+        RASQAL_FREE(char*, result);
         return 0;
     }
 
@@ -881,14 +907,14 @@ rasqal_world_set_default_generate_bnodeid_parameters(rasqal_world* world,
   if(prefix) {
     length = strlen(prefix);
     
-    prefix_copy = (char*)RASQAL_MALLOC(cstring, length + 1);
+    prefix_copy = RASQAL_MALLOC(char*, length + 1);
     if(!prefix_copy)
       return 1;
     memcpy(prefix_copy, prefix, length + 1);
   }
   
   if(world->default_generate_bnodeid_handler_prefix)
-    RASQAL_FREE(cstring, world->default_generate_bnodeid_handler_prefix);
+    RASQAL_FREE(char*, world->default_generate_bnodeid_handler_prefix);
 
   world->default_generate_bnodeid_handler_prefix = prefix_copy;
   world->default_generate_bnodeid_handler_prefix_length = length;
@@ -938,7 +964,7 @@ rasqal_world_default_generate_bnodeid_handler(void *user_data,
   rasqal_world *world = (rasqal_world*)user_data;
   int id;
   unsigned char *buffer;
-  int length;
+  size_t length;
   int tmpid;
 
   if(user_bnodeid)
@@ -956,7 +982,7 @@ rasqal_world_default_generate_bnodeid_handler(void *user_data,
   else
     length += 7; /* bnodeid */
   
-  buffer = (unsigned char*)RASQAL_MALLOC(cstring, length);
+  buffer = RASQAL_MALLOC(unsigned char*, length);
   if(!buffer)
     return NULL;
   if(world->default_generate_bnodeid_handler_prefix) {
@@ -1009,6 +1035,9 @@ rasqal_world_reset_now(rasqal_world* world)
   return 0;
 }
 
+#ifndef HAVE_GETTIMEOFDAY
+#define gettimeofday(x,y) rasqal_gettimeofday(x,y)
+#endif
 
 /**
  * rasqal_world_get_now_timeval:
@@ -1034,6 +1063,44 @@ rasqal_world_get_now_timeval(rasqal_world* world)
 }
 
 
+/**
+ * rasqal_world_set_warning_level:
+ * @world: world
+ * @warning_level: warning level 0..100
+ *
+ * Set the rasqal warning reporting level
+ *
+ * The warning levels used are as follows:
+ * <orderedlist>
+ * <listitem><para>Level 10 is used for serious warnings that may be errors.
+ * </para></listitem>
+ * <listitem><para>Level 30 is used for moderate style warnings.
+ * </para></listitem>
+ * <listitem><para>Level 90 is used for strict conformance warnings.
+ * </para></listitem>
+ * </orderedlist>
+ *
+ * When this method is called to set a warning level, only warnings
+ * of less than @warning_level are reported.  The default warning
+ * level is 50.
+ *
+ * Return value: non-0 on failure
+ */
+int
+rasqal_world_set_warning_level(rasqal_world* world,
+                               unsigned int warning_level)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
+
+  if(warning_level > RASQAL_WARNING_LEVEL_MAX)
+    return 1;
+  
+  world->warning_level = (rasqal_warning_level)warning_level;
+  
+  return 0;
+}
+
+
 /**
  * rasqal_free_memory:
  * @ptr: memory pointer
@@ -1073,7 +1140,7 @@ rasqal_free_memory(void *ptr)
 void*
 rasqal_alloc_memory(size_t size)
 {
-  return RASQAL_MALLOC(void, size);
+  return RASQAL_MALLOC(void*, size);
 }
 
 
@@ -1096,7 +1163,7 @@ rasqal_alloc_memory(size_t size)
 void*
 rasqal_calloc_memory(size_t nmemb, size_t size)
 {
-  return RASQAL_CALLOC(void, nmemb, size);
+  return RASQAL_CALLOC(void*, nmemb, size);
 }
 
 
