@@ -35,6 +35,10 @@ extern "C" {
 
 #ifdef RASQAL_INTERNAL
 
+#ifdef MAINTAINER_MODE
+#include <git-version.h>
+#endif
+
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 #define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index) \
   __attribute__((__format__(__printf__, string_index, first_to_check_index)))
@@ -42,6 +46,12 @@ extern "C" {
 #define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index)
 #endif
 
+#ifdef __GNUC__
+#define RASQAL_NORETURN __attribute__((noreturn)) 
+#else
+#define RASQAL_NORETURN
+#endif
+
 /* Can be over-ridden or undefined in a config.h file or -Ddefine */
 #ifndef RASQAL_INLINE
 #define RASQAL_INLINE inline
@@ -58,14 +68,14 @@ void* rasqal_sign_calloc(size_t nmemb, size_t size);
 void* rasqal_sign_realloc(void *ptr, size_t size);
 void rasqal_sign_free(void *ptr);
   
-#define RASQAL_MALLOC(type, size)   rasqal_sign_malloc(size)
-#define RASQAL_CALLOC(type, nmemb, size) rasqal_sign_calloc(nmemb, size)
-#define RASQAL_REALLOC(type, ptr, size) rasqal_sign_realloc(ptr, size)
-#define RASQAL_FREE(type, ptr)   rasqal_sign_free(ptr)
+#define RASQAL_MALLOC(type, size)   (type)rasqal_sign_malloc(size)
+#define RASQAL_CALLOC(type, nmemb, size) (type)rasqal_sign_calloc(nmemb, size)
+#define RASQAL_REALLOC(type, ptr, size) (type0rasqal_sign_realloc(ptr, size)
+#define RASQAL_FREE(type, ptr)   rasqal_sign_free((void*)ptr)
 
 #else
-#define RASQAL_MALLOC(type, size) malloc(size)
-#define RASQAL_CALLOC(type, size, count) calloc(size, count)
+#define RASQAL_MALLOC(type, size) (type)malloc(size)
+#define RASQAL_CALLOC(type, size, count) (type)calloc(size, count)
 #define RASQAL_FREE(type, ptr)   free((void*)ptr)
 
 #endif
@@ -201,7 +211,7 @@ typedef struct {
 
   int wildcard:1;
 
-  int distinct;
+  unsigned int distinct;
 } rasqal_projection;  
 
 
@@ -282,13 +292,19 @@ struct rasqal_graph_pattern_s {
 
   /* SELECT modifiers */
   rasqal_solution_modifier* modifier;
+
+  /* SILENT flag for SERVICE graph pattern */
+  unsigned int silent : 1;
+
+  /* SELECT graph pattern: sequence of #rasqal_data_graph */
+  raptor_sequence* data_graphs;
 };
 
 rasqal_graph_pattern* rasqal_new_basic_graph_pattern(rasqal_query* query, raptor_sequence* triples, int start_column, int end_column);
 rasqal_graph_pattern* rasqal_new_graph_pattern_from_sequence(rasqal_query* query, raptor_sequence* graph_patterns, rasqal_graph_pattern_operator op);
 rasqal_graph_pattern* rasqal_new_filter_graph_pattern(rasqal_query* query, rasqal_expression* expr);
 rasqal_graph_pattern* rasqal_new_let_graph_pattern(rasqal_query *query, rasqal_variable *var, rasqal_expression *expr);  
-rasqal_graph_pattern* rasqal_new_select_graph_pattern(rasqal_query *query, rasqal_projection* projection, rasqal_graph_pattern* where, rasqal_solution_modifier* modifier);
+rasqal_graph_pattern* rasqal_new_select_graph_pattern(rasqal_query *query, rasqal_projection* projection, raptor_sequence* data_graphs, rasqal_graph_pattern* where, rasqal_solution_modifier* modifier);
 rasqal_graph_pattern* rasqal_new_single_graph_pattern(rasqal_query* query, rasqal_graph_pattern_operator op, rasqal_graph_pattern* single);
 void rasqal_free_graph_pattern(rasqal_graph_pattern* gp);
 void rasqal_graph_pattern_adjust(rasqal_graph_pattern* gp, int offset);
@@ -363,7 +379,7 @@ struct rasqal_query_s {
   int usage; /* reference count - 1 for itself, plus for query_results */
   
   unsigned char* query_string;
-  int query_string_length; /* length including NULs */
+  size_t query_string_length; /* length including NULs */
 
   raptor_namespace_stack* namespaces;
 
@@ -372,9 +388,11 @@ struct rasqal_query_s {
   
   /* the query verb - in SPARQL terms: SELECT, CONSTRUCT, DESCRIBE or ASK */
   rasqal_query_verb verb;
-  
+
+  /* WAS: selects - sequence of rasqal_variable# */
+  raptor_sequence* unused10;     /* ... rasqal_variable* names only */
+
   /* sequences of ... */
-  raptor_sequence* selects;     /* ... rasqal_variable* names only */
   raptor_sequence* data_graphs; /* ... rasqal_data_graph*          */
   /* NOTE: Cannot assume that triples are in any of 
    * graph pattern use / query execution / document order 
@@ -385,13 +403,8 @@ struct rasqal_query_s {
   raptor_sequence* optional_triples; /* ... rasqal_triple*  SPARQL */
   raptor_sequence* describes;   /* ... rasqal_literal* (var or URIs) SPARQL */
 
-  /* DISTINCT mode:
-   * 0 if not given
-   * 1 if DISTINCT: ensure solutions are unique
-   * 2 if SPARQL REDUCED: permit elimination of some non-unique solutions 
-   * otherwise undefined
-   */
-  int distinct;
+  /* WAS: distinct 0..2; now projection->distinct */
+  unsigned int unused9;
 
   /* WAS: result limit LIMIT (>=0) or <0 if not given */
   int unused4;
@@ -399,18 +412,18 @@ struct rasqal_query_s {
   /* WAS: result offset OFFSET (>=0) or <0 if not given */
   int unused5;
 
-  /* flag: non-0 if '*' was seen after a verb (the appropriate list such as selects or constructs will be NULL) */
-  int wildcard;
+  /* WAS: wildcard flag; now projection->wildcard  */
+  int unused12;
 
   /* flag: non-0 if query has been prepared */
   int prepared;
 
   rasqal_variables_table* vars_table;
 
-  /* The number of selected variables: these are always the first
+  /* WAS: The number of selected variables: these are always the first
    * in the variables table and are the ones returend to the user.
    */
-  int select_variables_count;
+  int unused11;
 
   /* INTERNAL 2D array of:
    *   width (number of total variables)
@@ -473,7 +486,7 @@ struct rasqal_query_s {
    * is the name used for constructing a rasqal_query_formatter
    * from the results.
    */
-  const char* query_results_formatter_name;
+  char* query_results_formatter_name;
 
   /* flag: non-0 if EXPLAIN was given */
   int explain;
@@ -518,6 +531,9 @@ struct rasqal_query_s {
 
   /* INTERNAL flag: non-0 if user set a random seed via RASQAL_FEATURE_RAND_SEED */
   unsigned int user_set_rand : 1;
+
+  /* Variable projection (or NULL when invalid such as for ASK) */
+  rasqal_projection* projection;
 };
 
 
@@ -640,8 +656,11 @@ rasqal_rowsource* rasqal_new_project_rowsource(rasqal_world *world, rasqal_query
 /* rasqal_rowsource_rowsequence.c */
 rasqal_rowsource* rasqal_new_rowsequence_rowsource(rasqal_world *world, rasqal_query* query, rasqal_variables_table* vt, raptor_sequence* rows_seq, raptor_sequence* vars_seq);
 
+/* rasqal_rowsource_slice.c */
+rasqal_rowsource* rasqal_new_slice_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource* rowsource, int limit, int offset);
+
 /* rasqal_rowsource_sort.c */
-rasqal_rowsource* rasqal_new_sort_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource *rowsource);
+rasqal_rowsource* rasqal_new_sort_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource *rowsource, raptor_sequence* order_seq, int distinct);
 
 /* rasqal_rowsource_triples.c */
 rasqal_rowsource* rasqal_new_triples_rowsource(rasqal_world *world, rasqal_query* query, rasqal_triples_source* triples_source, raptor_sequence* triples, int start_column, int end_column);
@@ -714,16 +733,19 @@ typedef raptor_sequence* (*rasqal_rowsource_read_all_rows_func) (rasqal_rowsourc
 typedef int (*rasqal_rowsource_reset_func) (rasqal_rowsource* rowsource, void *user_data);
 
 
+/* bit flags */
+#define RASQAL_ROWSOURCE_REQUIRE_RESET (1 << 0)
+
 /**
- * rasqal_rowsource_set_preserve_func
+ * rasqal_rowsource_set_requirements_func
  * @user_data: user data
- * @preserve: flag
+ * @flags: bit flags
  *
- * Handler function for setting preserve binding states on a rowsource
+ * Handler function for one rowsource setting requirements on inner rowsources
  *
  * Return value: non-0 on failure
  */
-typedef int (*rasqal_rowsource_set_preserve_func) (rasqal_rowsource* rowsource, void *user_data, int preserve);
+typedef int (*rasqal_rowsource_set_requirements_func) (rasqal_rowsource* rowsource, void *user_data, unsigned int flags);
 
 
 /**
@@ -760,7 +782,7 @@ typedef int (*rasqal_rowsource_set_origin_func) (rasqal_rowsource* rowsource, vo
  * @read_row: read row handler - this or @read_all_rows required (V1)
  * @read_all_rows: read all rows handler - this or @read_row required (V1)
  * @reset: reset rowsource to starting state handler - optional (V1)
- * @set_preserve: set preserve flag handler - optional (V1)
+ * @set_requirements: set requirements flag handler - optional (V1)
  * @get_inner_rowsource: get inner rowsource handler - optional if has no inner rowsources (V1)
  * @set_origin: set origin (GRAPH) handler - optional (V1)
  *
@@ -777,7 +799,7 @@ typedef struct {
   rasqal_rowsource_read_row_func             read_row;
   rasqal_rowsource_read_all_rows_func        read_all_rows;
   rasqal_rowsource_reset_func                reset;
-  rasqal_rowsource_set_preserve_func         set_preserve;
+  rasqal_rowsource_set_requirements_func     set_requirements;
   rasqal_rowsource_get_inner_rowsource_func  get_inner_rowsource;
   rasqal_rowsource_set_origin_func           set_origin;
 } rasqal_rowsource_handler;
@@ -807,6 +829,17 @@ typedef struct {
 } rasqal_row_compatible;
 
 
+/* 
+ * Rowsource Internal flags
+ *
+ * RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS: need to save all rows in
+ * @rows_sequence for reset operation
+ *
+ * RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS: have saved rows ready for reply
+ */
+#define RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS  0x01
+#define RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS 0x02
+
 /**
  * rasqal_rowsource:
  * @world: rasqal world
@@ -901,7 +934,7 @@ int rasqal_rowsource_get_variable_offset_by_name(rasqal_rowsource *rowsource, co
 int rasqal_rowsource_copy_variables(rasqal_rowsource *dest_rowsource, rasqal_rowsource *src_rowsource);
 void rasqal_rowsource_print_row_sequence(rasqal_rowsource* rowsource,raptor_sequence* seq, FILE* fh);
 int rasqal_rowsource_reset(rasqal_rowsource* rowsource);
-int rasqal_rowsource_set_preserve(rasqal_rowsource* rowsource, int preserve);
+int rasqal_rowsource_set_requirements(rasqal_rowsource* rowsource, unsigned int requirement);
 rasqal_rowsource* rasqal_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource, int offset);
 int rasqal_rowsource_write(rasqal_rowsource *rowsource,  raptor_iostream *iostr);
 void rasqal_rowsource_print(rasqal_rowsource* rs, FILE* fh);
@@ -918,7 +951,7 @@ typedef void (*rasqal_query_results_finish_func)(rasqal_query_results_formatter*
 
 typedef int (*rasqal_query_results_write_func)(rasqal_query_results_formatter* formatter, raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
 
-typedef rasqal_rowsource* (*rasqal_query_results_get_rowsource_func)(rasqal_query_results_formatter* formatter, rasqal_world*, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri);
+typedef rasqal_rowsource* (*rasqal_query_results_get_rowsource_func)(rasqal_query_results_formatter* formatter, rasqal_world*, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri, unsigned int flags);
 
 typedef int (*rasqal_query_results_recognise_syntax_func)(struct rasqal_query_results_format_factory_s* factory, const unsigned char *buffer, size_t len, const unsigned char *identifier, const unsigned char *suffix, const char *mime_type);
 
@@ -965,6 +998,9 @@ struct rasqal_query_results_formatter_s {
   void* context;
 };
 
+/* rasqal_results_formats.c */
+rasqal_rowsource* rasqal_query_results_formatter_get_read_rowsource(rasqal_world *world, raptor_iostream *iostr, rasqal_query_results_formatter* formatter, rasqal_variables_table* vars_table, raptor_uri *base_uri, unsigned int flags);
+
 
 typedef struct {
   rasqal_world *world;
@@ -1069,9 +1105,9 @@ int rasqal_query_constraints_has_qname(rasqal_query* gp);
 int rasqal_query_expand_graph_pattern_constraints_qnames(rasqal_query* rq, rasqal_graph_pattern* gp);
 int rasqal_query_expand_query_constraints_qnames(rasqal_query* rq);
 int rasqal_query_build_anonymous_variables(rasqal_query* rq);
-int rasqal_query_expand_wildcards(rasqal_query* rq);
-int rasqal_query_remove_duplicate_select_vars(rasqal_query* rq);
-int rasqal_query_build_variables_use(rasqal_query* query);
+int rasqal_query_expand_wildcards(rasqal_query* rq, rasqal_projection* projection);
+int rasqal_query_remove_duplicate_select_vars(rasqal_query* rq, rasqal_projection* projection);
+int rasqal_query_build_variables_use(rasqal_query* query, rasqal_projection* projection);
 int rasqal_query_prepare_common(rasqal_query *query);
 int rasqal_query_merge_graph_patterns(rasqal_query* query, rasqal_graph_pattern* gp, void* data);
 int rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp, rasqal_graph_pattern *src_gp);
@@ -1081,7 +1117,8 @@ int rasqal_graph_pattern_move_constraints(rasqal_graph_pattern* dest_gp, rasqal_
 rasqal_literal* rasqal_new_string_literal_node(rasqal_world*, const unsigned char *string, const char *language, raptor_uri *datatype);
 int rasqal_literal_as_boolean(rasqal_literal* literal, int* error_p);
 int rasqal_literal_as_integer(rasqal_literal* l, int* error_p);
-double rasqal_literal_as_floating(rasqal_literal* l, int* error_p);
+double rasqal_literal_as_double(rasqal_literal* l, int* error_p);
+float rasqal_literal_as_floating(rasqal_literal* l, int* error_p);
 raptor_uri* rasqal_literal_as_uri(rasqal_literal* l);
 int rasqal_literal_string_to_native(rasqal_literal *l, int flags);
 int rasqal_literal_has_qname(rasqal_literal* l);
@@ -1167,7 +1204,7 @@ void rasqal_redland_finish(void);
 #endif
 
 rasqal_triple* raptor_statement_as_rasqal_triple(rasqal_world* world, const raptor_statement *statement);
-int rasqal_raptor_triple_match(rasqal_world* world, rasqal_triple *triple, rasqal_triple *match, rasqal_triple_parts parts);
+int rasqal_raptor_triple_match(rasqal_world* world, rasqal_triple *triple, rasqal_triple *match, unsigned int parts);
 
 
 /* rasqal_general.c */
@@ -1240,11 +1277,15 @@ rasqal_variable* rasqal_query_get_variable_by_offset(rasqal_query* query, int id
 const rasqal_query_execution_factory* rasqal_query_get_engine_by_name(const char* name);
 int rasqal_query_variable_is_bound(rasqal_query* query, rasqal_variable* v);
 rasqal_triple_parts rasqal_query_variable_bound_in_triple(rasqal_query* query, rasqal_variable* v, int column);
+int rasqal_query_store_select_query(rasqal_query* query, rasqal_projection* projection, raptor_sequence* data_graphs, rasqal_graph_pattern* where_gp, rasqal_solution_modifier* modifier);
+rasqal_projection* rasqal_query_get_projection(rasqal_query* query);
+int rasqal_query_set_projection(rasqal_query* query, rasqal_projection* projection);
 
 /* rasqal_query_results.c */
 int rasqal_init_query_results(void);
 void rasqal_finish_query_results(void);
 int rasqal_query_results_execute_with_engine(rasqal_query_results* query_results, const rasqal_query_execution_factory* factory, int store_results);
+int rasqal_query_check_limit_offset_core(int result_offset, int limit, int offset);
 int rasqal_query_check_limit_offset(rasqal_query* query, int result_offset);
 void rasqal_query_results_remove_query_reference(rasqal_query_results* query_results);
 rasqal_variables_table* rasqal_query_results_get_variables_table(rasqal_query_results* query_results);
@@ -1292,7 +1333,7 @@ void rasqal_row_set_values_from_variables_table(rasqal_row* row, rasqal_variable
 int rasqal_row_set_order_size(rasqal_row *row, int order_size);
 int rasqal_row_expand_size(rasqal_row *row, int size);
 int rasqal_row_bind_variables(rasqal_row* row, rasqal_variables_table* vars_table);
-
+raptor_sequence* rasqal_row_sequence_copy(raptor_sequence *seq);
 
 /* rasqal_row_compatible.c */
 rasqal_row_compatible* rasqal_new_row_compatible(rasqal_variables_table* vt, rasqal_rowsource *first_rowsource, rasqal_rowsource *second_rowsource);
@@ -1454,18 +1495,21 @@ struct rasqal_algebra_node_s {
   /* types PROJECT, DISTINCT, REDUCED
    * FIXME: sequence of solution mappings */
 
-  /* types PROJECT, SLICE, AGGREGATION: sequence of #rasqal_variable */
+  /* types PROJECT, AGGREGATION: sequence of #rasqal_variable */
   raptor_sequence* vars_seq;
 
-  /* type SLICE: start and length */
-  unsigned int start;
-  unsigned int length;
+  /* type SLICE: limit and offset rows */
+  unsigned int limit;
+  unsigned int offset;
 
   /* type GRAPH */
   rasqal_literal *graph;
 
   /* type LET */
   rasqal_variable *var;
+
+  /* type ORDERBY */
+  int distinct;
 };
 typedef struct rasqal_algebra_node_s rasqal_algebra_node;
 
@@ -1528,7 +1572,8 @@ rasqal_algebra_node* rasqal_new_triples_algebra_node(rasqal_query* query, raptor
 rasqal_algebra_node* rasqal_new_2op_algebra_node(rasqal_query* query, rasqal_algebra_node_operator op, rasqal_algebra_node* node1, rasqal_algebra_node* node2);
 rasqal_algebra_node* rasqal_new_leftjoin_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, rasqal_algebra_node* node2, rasqal_expression* expr);
 rasqal_algebra_node* rasqal_new_join_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, rasqal_algebra_node* node2, rasqal_expression* expr);
-rasqal_algebra_node* rasqal_new_orderby_algebra_node(rasqal_query* query, rasqal_algebra_node* node, raptor_sequence* seq);
+rasqal_algebra_node* rasqal_new_orderby_algebra_node(rasqal_query* query, rasqal_algebra_node* node, raptor_sequence* seq, int distinct);
+rasqal_algebra_node* rasqal_new_slice_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, int limit, int offset);
 rasqal_algebra_node* rasqal_new_project_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, raptor_sequence* vars_seq);
 rasqal_algebra_node* rasqal_new_graph_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, rasqal_literal *graph);
 rasqal_algebra_node* rasqal_new_let_algebra_node(rasqal_query* query, rasqal_variable *var, rasqal_expression *expr);
@@ -1538,29 +1583,29 @@ rasqal_algebra_node* rasqal_new_having_algebra_node(rasqal_query* query,rasqal_a
 
 void rasqal_free_algebra_node(rasqal_algebra_node* node);
 rasqal_algebra_node_operator rasqal_algebra_node_get_operator(rasqal_algebra_node* node);
-const char* rasqal_algebra_node_operator_as_string(rasqal_algebra_node_operator op);
+const char* rasqal_algebra_node_operator_as_counted_string(rasqal_algebra_node_operator op, size_t* length_p);
 int rasqal_algebra_algebra_node_write(rasqal_algebra_node *node, raptor_iostream* iostr);
 int rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh);
 int rasqal_algebra_node_visit(rasqal_query *query, rasqal_algebra_node* node, rasqal_algebra_node_visit_fn fn, void *user_data);
 rasqal_algebra_node* rasqal_algebra_query_to_algebra(rasqal_query* query);
-rasqal_algebra_node* rasqal_algebra_query_add_group_by(rasqal_query* query, rasqal_algebra_node* node);
-rasqal_algebra_node* rasqal_algebra_query_add_modifiers(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_group_by(rasqal_query* query, rasqal_algebra_node* node, rasqal_solution_modifier* modifier);
+rasqal_algebra_node* rasqal_algebra_query_add_orderby(rasqal_query* query, rasqal_algebra_node* node, rasqal_projection* projection, rasqal_solution_modifier* modifier);
+rasqal_algebra_node* rasqal_algebra_query_add_slice(rasqal_query* query, rasqal_algebra_node* node, rasqal_solution_modifier* modifier);
 rasqal_algebra_node* rasqal_algebra_query_add_aggregation(rasqal_query* query, rasqal_algebra_aggregate* ae, rasqal_algebra_node* node);
-rasqal_algebra_node* rasqal_algebra_query_add_projection(rasqal_query* query, rasqal_algebra_node* node);
-rasqal_algebra_node* rasqal_algebra_query_add_distinct(rasqal_query* query, rasqal_algebra_node* node);
-rasqal_algebra_node* rasqal_algebra_query_add_having(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_projection(rasqal_query* query, rasqal_algebra_node* node, rasqal_projection* projection);
+rasqal_algebra_node* rasqal_algebra_query_add_construct_projection(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_distinct(rasqal_query* query, rasqal_algebra_node* node, rasqal_projection* projection);
+rasqal_algebra_node* rasqal_algebra_query_add_having(rasqal_query* query, rasqal_algebra_node* node, rasqal_solution_modifier* modifier);
 int rasqal_algebra_node_is_empty(rasqal_algebra_node* node);
 
-rasqal_algebra_aggregate* rasqal_algebra_query_prepare_aggregates(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_aggregate* rasqal_algebra_query_prepare_aggregates(rasqal_query* query, rasqal_algebra_node* node, rasqal_projection* projection, rasqal_solution_modifier* modifier);
 void rasqal_free_algebra_aggregate(rasqal_algebra_aggregate* ae);
 
 /* rasqal_variable.c */
 rasqal_variables_table* rasqal_new_variables_table_from_variables_table(rasqal_variables_table* vt);
 rasqal_variable* rasqal_variables_table_get(rasqal_variables_table* vt, int idx);
-rasqal_variable* rasqal_variables_table_get_by_name(rasqal_variables_table* vt, const unsigned char *name);
 rasqal_literal* rasqal_variables_table_get_value(rasqal_variables_table* vt, int idx);
-int rasqal_variables_table_has(rasqal_variables_table* vt, const unsigned char *name);
-int rasqal_variables_table_set(rasqal_variables_table* vt, const unsigned char *name, rasqal_literal* value);
+int rasqal_variables_table_set(rasqal_variables_table* vt, rasqal_variable_type type, const unsigned char *name, rasqal_literal* value);
 int rasqal_variables_table_get_named_variables_count(rasqal_variables_table* vt);
 int rasqal_variables_table_get_anonymous_variables_count(rasqal_variables_table* vt);
 int rasqal_variables_table_get_total_variables_count(rasqal_variables_table* vt);
@@ -1652,7 +1697,7 @@ const char* rasqal_engine_error_as_string(rasqal_engine_error error);
 rasqal_map* rasqal_engine_new_rowsort_map(int is_distinct, int compare_flags, raptor_sequence* order_conditions_sequence);
 int rasqal_engine_rowsort_map_add_row(rasqal_map* map, rasqal_row* row);
 raptor_sequence* rasqal_engine_rowsort_map_to_sequence(rasqal_map* map, raptor_sequence* seq);
-int rasqal_engine_rowsort_calculate_order_values(rasqal_query* query, rasqal_row* row);
+int rasqal_engine_rowsort_calculate_order_values(rasqal_query* query, raptor_sequence* order_seq, rasqal_row* row);
 
 
 /* rasqal_engine_algebra.c */
@@ -1660,6 +1705,12 @@ int rasqal_engine_rowsort_calculate_order_values(rasqal_query* query, rasqal_row
 /* New query engine based on executing over query algebra */
 extern const rasqal_query_execution_factory rasqal_query_engine_algebra;
 
+/* rasqal_iostream.c */
+raptor_iostream* rasqal_new_iostream_from_stringbuffer(raptor_world *raptor_world_ptr, raptor_stringbuffer* sb);
+
+/* rasqal_service.c */
+rasqal_rowsource* rasqal_service_execute_as_rowsource(rasqal_service* svc, rasqal_variables_table* vars_table);
+
 /* rasqal_triples_source.c */
 void rasqal_triples_source_error_handler(rasqal_query* rdf_query, raptor_locator* locator, const char* message);
 
@@ -1681,6 +1732,7 @@ int rasqal_bindings_print(rasqal_bindings* bindings, FILE* fh);
 rasqal_projection* rasqal_new_projection(rasqal_query* query, raptor_sequence* variables, int wildcard, int distinct);
 void rasqal_free_projection(rasqal_projection* projection);
 raptor_sequence* rasqal_projection_get_variables_sequence(rasqal_projection* projection);
+int rasqal_projection_add_variable(rasqal_projection* projection, rasqal_variable* var);
 
 
 /* rasqal_solution_modifier.c */
