@@ -71,7 +71,7 @@ typedef struct {
   /* genid base for mapping user bnodes */
   unsigned char* mapped_id_base;
   /* length of above string */
-  int mapped_id_base_len;
+  size_t mapped_id_base_len;
 } rasqal_raptor_triples_source_user_data;
 
 
@@ -92,8 +92,8 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   /* subject */
   if(statement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->subject->value.blank.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->subject->value.blank.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->subject->value.blank.string, len + 1);
     s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
@@ -109,13 +109,13 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
   if(statement->object->type == RAPTOR_TERM_TYPE_LITERAL) {
     char *language = NULL;
 
-    len = strlen((char*)statement->object->value.literal.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->object->value.literal.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->object->value.literal.string, len + 1);
 
     if(statement->object->value.literal.language) {
-      len = strlen((const char*)statement->object->value.literal.language);
-      language = (char*)RASQAL_MALLOC(cstring, len + 1);
+      len = statement->object->value.literal.language_len;
+      language = RASQAL_MALLOC(char*, len + 1);
       memcpy(language, statement->object->value.literal.language, len + 1);
     }
 
@@ -126,8 +126,8 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     o = rasqal_new_string_literal(world, new_str, language, uri, NULL);
   } else if(statement->object->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->object->value.blank.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->object->value.blank.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->object->value.blank.string, len + 1);
     o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
@@ -148,8 +148,7 @@ rasqal_raptor_statement_handler(void *user_data,
   
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
-  triple = (rasqal_raptor_triple*)RASQAL_MALLOC(rasqal_raptor_triple,
-                                                sizeof(rasqal_raptor_triple));
+  triple = RASQAL_MALLOC(rasqal_raptor_triple*, sizeof(rasqal_raptor_triple));
   triple->next = NULL;
   triple->triple = raptor_statement_as_rasqal_triple(rtsc->query->world,
                                                      statement);
@@ -181,9 +180,8 @@ rasqal_raptor_generate_id_handler(void *user_data,
     unsigned char *mapped_id;
     size_t user_bnodeid_len = strlen((const char*)user_bnodeid);
     
-    mapped_id = (unsigned char*)RASQAL_MALLOC(cstring, 
-                                              rtsc->mapped_id_base_len + 1 + 
-                                              user_bnodeid_len + 1);
+    mapped_id = RASQAL_MALLOC(unsigned char*, 
+                              rtsc->mapped_id_base_len + 1 + user_bnodeid_len + 1);
     memcpy(mapped_id, rtsc->mapped_id_base, rtsc->mapped_id_base_len);
     mapped_id[rtsc->mapped_id_base_len] = '_';
     memcpy(mapped_id + rtsc->mapped_id_base_len + 1,
@@ -241,9 +239,8 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
     rtsc->sources_count = 0;
   
   if(rtsc->sources_count)
-    rtsc->source_literals = (rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr,
-                                                            rtsc->sources_count,
-                                                            sizeof(rasqal_literal*));
+    rtsc->source_literals = RASQAL_CALLOC(rasqal_literal**, rtsc->sources_count,
+                                          sizeof(rasqal_literal*));
   else
     rtsc->source_literals = NULL;
 
@@ -319,7 +316,7 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
 
     /* This is freed in rasqal_raptor_free_triples_source() */
     /* rasqal_free_literal(rtsc->source_literal); */
-    RASQAL_FREE(cstring, rtsc->mapped_id_base);
+    RASQAL_FREE(char*, rtsc->mapped_id_base);
     
     if(rdf_query->failed) {
       rasqal_raptor_free_triples_source(user_data);
@@ -348,7 +345,7 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
  * @world: rasqal_world object
  * @triple: #rasqal_triple to match against
  * @match: #rasqal_triple with wildcards
- * @parts; parts of the triple to match
+ * @parts: parts of the triple to match - XOR of #rasqal_triple_parts bits
  * .
  * 
  * Match a rasqal_triple against a rasqal_triple with NULL
@@ -360,7 +357,7 @@ int
 rasqal_raptor_triple_match(rasqal_world* world,
                            rasqal_triple *triple,
                            rasqal_triple *match,
-                           rasqal_triple_parts parts)
+                           unsigned int parts)
 {
   int rc = 0;
   
@@ -432,7 +429,7 @@ rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_data,
 {
   rasqal_raptor_triples_source_user_data* rtsc;
   rasqal_raptor_triple *triple;
-  rasqal_triple_parts parts = RASQAL_TRIPLE_SPO;
+  unsigned int parts = RASQAL_TRIPLE_SPO;
   
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
@@ -497,6 +494,8 @@ typedef struct {
 
   /* parts of the triple above to match: always (S,P,O) sometimes C */
   rasqal_triple_parts parts;
+
+  unsigned int bind_parts;
 } rasqal_raptor_triples_match_context;
 
 
@@ -604,8 +603,38 @@ rasqal_raptor_next_match(struct rasqal_triples_match_s* rtm, void *user_data)
 
   rtmc = (rasqal_raptor_triples_match_context*)rtm->user_data;
 
+#if 0
+  if(rtmc->bind_parts & RASQAL_TRIPLE_SUBJECT) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->subject);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_PREDICATE) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->predicate);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_OBJECT) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->object);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_ORIGIN) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->origin);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+#endif
+
   while(rtmc->cur) {
     rtmc->cur = rtmc->cur->next;
+#ifdef RASQAL_DEBUG
+    if(!rtmc->cur) {
+      RASQAL_DEBUG1("triple match ended when matching ");
+      rasqal_triple_print(&rtmc->match, stderr);
+      fputc('\n', stderr);
+    }
+#endif
     if(rtmc->cur &&
        rasqal_raptor_triple_match(rtm->world, rtmc->cur->triple, &rtmc->match,
                                   rtmc->parts))
@@ -664,7 +693,7 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   rtm->is_end = rasqal_raptor_is_end;
   rtm->finish = rasqal_raptor_finish_triples_match;
 
-  rtmc = (rasqal_raptor_triples_match_context*)RASQAL_CALLOC(rasqal_raptor_triples_match_context, 1, sizeof(rasqal_raptor_triples_match_context));
+  rtmc = RASQAL_CALLOC(rasqal_raptor_triples_match_context*, 1, sizeof(*rtmc));
   if(!rtmc)
     return -1;
 
@@ -673,12 +702,18 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   rtmc->source_context = rtsc;
   rtmc->cur = rtsc->head;
   
+  /* Parts we bind */
+  rtmc->bind_parts = m->parts;
+
   /* at least one of the triple terms is a variable and we need to
    * do a triplesMatching() over the list of stored raptor_statements
    */
 
   if((var = rasqal_literal_as_variable(t->subject))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_SUBJECT)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.subject = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.subject = rasqal_new_literal_from_literal(t->subject);
@@ -687,7 +722,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   
 
   if((var = rasqal_literal_as_variable(t->predicate))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_PREDICATE)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.predicate = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.predicate = rasqal_new_literal_from_literal(t->predicate);
@@ -696,7 +734,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   
 
   if((var = rasqal_literal_as_variable(t->object))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_OBJECT)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.object = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.object = rasqal_new_literal_from_literal(t->object);
@@ -707,7 +748,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
 
   if(t->origin) {
     if((var = rasqal_literal_as_variable(t->origin))) {
-      if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_ORIGIN)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
         rtmc->match.origin = rasqal_new_literal_from_literal(var->value);
     } else
       rtmc->match.origin = rasqal_new_literal_from_literal(t->origin);
