@@ -81,6 +81,9 @@ typedef struct
   /* dataset graph offset */
   int dg_offset;
   
+  /* number of graphs total */
+  int dg_size;
+  
   /* row offset for read_row() */
   int offset;
 
@@ -89,9 +92,6 @@ typedef struct
 } rasqal_graph_rowsource_context;
 
 
-static int rasqal_graph_rowsource_reset(rasqal_rowsource* rowsource, void *user_data);
-
-
 static int
 rasqal_graph_next_dg(rasqal_graph_rowsource_context *con) 
 {
@@ -138,7 +138,28 @@ rasqal_graph_next_dg(rasqal_graph_rowsource_context *con)
 static int
 rasqal_graph_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
 {
-  return rasqal_graph_rowsource_reset(rowsource, user_data);
+  rasqal_graph_rowsource_context *con;
+  raptor_sequence* seq;
+
+  con = (rasqal_graph_rowsource_context*)user_data;
+  
+  seq = rasqal_query_get_data_graph_sequence(rowsource->query);
+  if(!seq)
+    return 1;
+
+  con->dg_size = raptor_sequence_size(seq);
+  
+  con->finished = 0;
+  con->dg_offset = -1;
+  con->offset = 0;
+
+  /* Do not care if finished at this stage (it is not an
+   * error). rasqal_graph_rowsource_read_row() will deal with
+   * returning NULL for an empty result.
+   */
+  rasqal_graph_next_dg(con);
+
+  return 0;
 }
 
 
@@ -219,8 +240,10 @@ rasqal_graph_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
       
       /* Put GRAPH variable value (or NULL) first in result row */
       nrow->values[0] = rasqal_new_literal_from_literal(con->var->value);
+
+      /* Copy (size-1) remaining variables from input row */
       for(i = 0; i < row->size; i++)
-        nrow->values[i+1] = rasqal_new_literal_from_literal(row->values[i]);
+        nrow->values[i + 1] = rasqal_new_literal_from_literal(row->values[i]);
       rasqal_free_row(row);
       row = nrow;
     }
@@ -246,17 +269,6 @@ rasqal_graph_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_graph_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                    void *user_data, int preserve)
-{
-  rasqal_graph_rowsource_context *con;
-  con = (rasqal_graph_rowsource_context*)user_data;
-
-  return rasqal_rowsource_set_preserve(con->rowsource, preserve);
-}
-
-
 static rasqal_rowsource*
 rasqal_graph_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource,
                                            void *user_data, int offset)
@@ -279,7 +291,7 @@ static const rasqal_rowsource_handler rasqal_graph_rowsource_handler = {
   /* .read_row =         */ rasqal_graph_rowsource_read_row,
   /* .read_all_rows =    */ NULL,
   /* .reset =            */ rasqal_graph_rowsource_reset,
-  /* .set_preserve =     */ rasqal_graph_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ rasqal_graph_rowsource_get_inner_rowsource,
   /* .set_origin =       */ NULL,
 };
@@ -310,7 +322,7 @@ rasqal_new_graph_rowsource(rasqal_world *world,
   if(!world || !query || !rowsource || !var)
     return NULL;
   
-  con = (rasqal_graph_rowsource_context*)RASQAL_CALLOC(rasqal_graph_rowsource_context, 1, sizeof(rasqal_graph_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_graph_rowsource_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
