@@ -84,7 +84,7 @@ rasqal_project_rowsource_ensure_variables(rasqal_rowsource* rowsource,
 
   size = raptor_sequence_size(con->projection_variables);
 
-  con->projection = (int*)RASQAL_MALLOC(array, sizeof(int*) * size);
+  con->projection = RASQAL_MALLOC(int*, sizeof(int*) * size);
   if(!con->projection)
     return 1;
   
@@ -208,17 +208,6 @@ rasqal_project_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_project_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                      void *user_data, int preserve)
-{
-  rasqal_project_rowsource_context *con;
-  con = (rasqal_project_rowsource_context*)user_data;
-
-  return rasqal_rowsource_set_preserve(con->rowsource, preserve);
-}
-
-
 static rasqal_rowsource*
 rasqal_project_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource,
                                              void *user_data, int offset)
@@ -241,7 +230,7 @@ static const rasqal_rowsource_handler rasqal_project_rowsource_handler = {
   /* .read_row =         */ rasqal_project_rowsource_read_row,
   /* .read_all_rows =    */ NULL,
   /* .reset =            */ rasqal_project_rowsource_reset,
-  /* .set_preserve =     */ rasqal_project_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ rasqal_project_rowsource_get_inner_rowsource,
   /* .set_origin =       */ NULL,
 };
@@ -272,7 +261,7 @@ rasqal_new_project_rowsource(rasqal_world *world,
   if(!world || !query || !rowsource || !projection_variables)
     goto fail;
   
-  con = (rasqal_project_rowsource_context*)RASQAL_CALLOC(rasqal_project_rowsource_context, 1, sizeof(rasqal_project_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_project_rowsource_context*, 1, sizeof(*con));
   if(!con)
     goto fail;
 
@@ -374,7 +363,8 @@ main(int argc, char *argv[])
   for(i = 0 ; i < EXPECTED_COLUMNS_COUNT; i++) {
     rasqal_variable* v;
     unsigned const char* name=(unsigned const char*)project_1_var_names[i];
-    v = rasqal_variables_table_get_by_name(vt, name);
+    v = rasqal_variables_table_get_by_name(vt, RASQAL_VARIABLE_TYPE_NORMAL,
+                                           name);
     if(v) {
       v = rasqal_new_variable_from_variable(v);
       raptor_sequence_push(projection_seq, v);
