@@ -73,15 +73,6 @@ typedef struct
   /* number of variables used in variables table  */
   int size;
   
-  /* bound-in array index[index into vars table] = column */
-  int *bound_in;
-
-  /* number of variables in bound_in[] array - may be more than 'size' */
-  int bound_in_size;
-  
-  /* preserve bindings when all rows are finished - for optional mostly */
-  int preserve_on_all_finished;
-
   /* GRAPH origin to use */
   rasqal_literal *origin;
 } rasqal_triples_rowsource_context;
@@ -90,28 +81,31 @@ typedef struct
 static int
 rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
 {
+  rasqal_query *query = rowsource->query;
   rasqal_triples_rowsource_context *con;
   int column;
-  int *bound_in = NULL;
   int rc = 0;
   int size;
   int i;
   
   con = (rasqal_triples_rowsource_context*)user_data;
 
-  bound_in = con->bound_in;
-  size = con->bound_in_size;
+  size = rasqal_variables_table_get_total_variables_count(query->vars_table);
   
   /* Construct the ordered projection of the variables set by these triples */
   con->size = 0;
   for(i = 0; i < size; i++) {
-    column = bound_in[i];
-    if(column >= con->start_column && column <= con->end_column) {
-      rasqal_variable *v;
-      v = rasqal_variables_table_get(rowsource->vars_table, i);
-      if(raptor_sequence_push(rowsource->variables_sequence, v))
-        return -1;
-      con->size++;
+    rasqal_variable *v;
+    v = rasqal_variables_table_get(rowsource->vars_table, i);
+    
+    for(column = con->start_column; column <= con->end_column; column++) {
+      if(rasqal_query_variable_bound_in_triple(query, v, column)) {
+          v = rasqal_new_variable_from_variable(v);
+          if(raptor_sequence_push(rowsource->variables_sequence, v))
+            return -1;
+          con->size++;
+          break; /* end column search loop */
+        }
     }
   }
 
@@ -129,15 +123,15 @@ rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
     t = (rasqal_triple*)raptor_sequence_get_at(con->triples, column);
     
     if((v = rasqal_literal_as_variable(t->subject)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_SUBJECT)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_SUBJECT);
     
     if((v = rasqal_literal_as_variable(t->predicate)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_PREDICATE)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_PREDICATE);
     
     if((v = rasqal_literal_as_variable(t->object)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_variable_bound_in_triple(query, v, column) & RASQAL_TRIPLE_OBJECT)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_OBJECT);
 
     RASQAL_DEBUG4("triple pattern column %d has parts %s (%d)\n", column,
@@ -180,9 +174,6 @@ rasqal_triples_rowsource_finish(rasqal_rowsource* rowsource, void *user_data)
     RASQAL_FREE(rasqal_triple_meta, con->triple_meta);
   }
 
-  if(con->bound_in)
-    RASQAL_FREE(rasqal_bound_in, con->bound_in);
-
   if(con->origin)
     rasqal_free_literal(con->origin);
 
@@ -228,30 +219,6 @@ rasqal_triples_rowsource_get_next_row(rasqal_rowsource* rowsource,
       RASQAL_DEBUG2("end of pattern triples match for column %d\n",
                     con->column);
 
-#if 0
-      if(con->preserve_on_all_finished && 
-         con->column == con->end_column) {
-        int is_finished = 1;
-        int i;
-          
-        RASQAL_DEBUG1("CHECKING ALL TRIPLE PATTERNS FINISHED\n");
-
-        for(i = con->start_column; i <= con->end_column; i++) {
-          rasqal_triple_meta *m2;
-          m2 = &con->triple_meta[con->column - con->start_column];
-          if(!rasqal_triples_match_is_end(m2->triples_match)) {
-            is_finished = 0;
-            break;
-          }
-        }
-        if(is_finished) {
-          RASQAL_DEBUG1("end of all pattern triples matches\n");
-          error = RASQAL_ENGINE_FINISHED;
-          break;
-        }
-      }
-#endif
-
       /* reset this column and move to next match in previous column */
       rasqal_reset_triple_meta(m);
       con->column--;
@@ -353,11 +320,8 @@ static raptor_sequence*
 rasqal_triples_rowsource_read_all_rows(rasqal_rowsource* rowsource,
                                        void *user_data)
 {
-  rasqal_triples_rowsource_context *con;
   raptor_sequence *seq = NULL;
   
-  con = (rasqal_triples_rowsource_context*)user_data;
-
   return seq;
 }
 
@@ -382,19 +346,6 @@ rasqal_triples_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_triples_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                      void *user_data, int preserve)
-{
-  rasqal_triples_rowsource_context *con;
-
-  con = (rasqal_triples_rowsource_context*)user_data;
-  con->preserve_on_all_finished = preserve;
-
-  return 0;
-}
-
-
 static int
 rasqal_triples_rowsource_set_origin(rasqal_rowsource *rowsource,
                                     void *user_data,
@@ -429,13 +380,13 @@ static const rasqal_rowsource_handler rasqal_triples_rowsource_handler = {
   /* .read_row = */ rasqal_triples_rowsource_read_row,
   /* .read_all_rows = */ rasqal_triples_rowsource_read_all_rows,
   /* .reset = */ rasqal_triples_rowsource_reset,
-  /* .set_preserve = */ rasqal_triples_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ rasqal_triples_rowsource_set_origin
 };
 
 
-/*
+/**
  * rasqal_new_triples_rowsource:
  * @world: world object
  * @query: query object
@@ -447,8 +398,6 @@ static const rasqal_rowsource_handler rasqal_triples_rowsource_handler = {
  *
  * INTERNAL - create a new triples rowsource
  *
- * NOTE @bound_in becomes owned by the new rowsource
- *
  * Return value: new triples rowsource or NULL on failure
  */
 rasqal_rowsource*
@@ -456,41 +405,31 @@ rasqal_new_triples_rowsource(rasqal_world *world,
                              rasqal_query *query,
                              rasqal_triples_source* triples_source,
                              raptor_sequence* triples,
-                             int start_column, int end_column,
-                             int *bound_in, int bound_in_size)
+                             int start_column, int end_column)
 {
   rasqal_triples_rowsource_context *con;
   int flags = 0;
 
-  if(!world || !query || !triples_source) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  if(!world || !query || !triples_source)
     return NULL;
-  }
 
-  if(!triples) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  if(!triples)
     return rasqal_new_empty_rowsource(world, query);
-  }
   
-  con = (rasqal_triples_rowsource_context*)RASQAL_CALLOC(rasqal_triples_rowsource_context, 1, sizeof(rasqal_triples_rowsource_context));
-  if(!con) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  con = RASQAL_CALLOC(rasqal_triples_rowsource_context*, 1, sizeof(*con));
+  if(!con)
     return NULL;
-  }
 
   con->triples_source = triples_source;
   con->triples = triples;
   con->start_column = start_column;
   con->end_column = end_column;
   con->column = -1;
-  con->bound_in = bound_in;
-  con->bound_in_size = bound_in_size;
 
   con->triples_count = con->end_column - con->start_column + 1;
 
-  con->triple_meta = (rasqal_triple_meta*)RASQAL_CALLOC(rasqal_triple_meta,
-                                                        con->triples_count,
-                                                        sizeof(rasqal_triple_meta));
+  con->triple_meta = RASQAL_CALLOC(rasqal_triple_meta*, con->triples_count,
+                                   sizeof(rasqal_triple_meta));
   if(!con->triple_meta) {
     rasqal_triples_rowsource_finish(NULL, con);
     return NULL;
@@ -545,8 +484,6 @@ main(int argc, char *argv[])
 #define OBJECT_STRING "object"
   raptor_uri* s_uri = NULL;
   raptor_uri* p_uri = NULL;
-  int size;
-  int *bound_in;
   const char *data_file;
   
   if((data_file = getenv("NT_DATA_FILE"))) {
@@ -568,7 +505,7 @@ main(int argc, char *argv[])
   query = rasqal_new_query(world, "sparql", NULL);
   
   data_string = raptor_uri_filename_to_uri_string(data_file);
-  query_string = (unsigned char*)RASQAL_MALLOC(cstring, strlen((const char*)data_string)+strlen(query_format)+1);
+  query_string = RASQAL_MALLOC(unsigned char*, strlen((const char*)data_string) + strlen(query_format) + 1);
   sprintf((char*)query_string, query_format, data_string);
   raptor_free_memory(data_string);
   
@@ -593,7 +530,7 @@ main(int argc, char *argv[])
     goto tidy;
   }
   
-  RASQAL_FREE(cstring, query_string);
+  RASQAL_FREE(char*, query_string);
   query_string = NULL;
 
   triples = rasqal_query_get_triple_sequence(query);
@@ -602,19 +539,8 @@ main(int argc, char *argv[])
 
   triples_source = rasqal_new_triples_source(query);
   
-  size = rasqal_variables_table_get_named_variables_count(query->vars_table);
-  bound_in = rasqal_query_triples_build_bound_in(query, size,
-                                                 start_column,
-                                                 end_column);
-  if(!bound_in) {
-    fprintf(stderr, "%s: failed to create bound_in\n", program);
-    failures++;
-    goto tidy;
-  }
-
   rowsource = rasqal_new_triples_rowsource(world, query, triples_source,
-                                           triples, start_column, end_column,
-                                           bound_in, size);
+                                           triples, start_column, end_column);
   if(!rowsource) {
     fprintf(stderr, "%s: failed to create triples rowsource\n", program);
     failures++;
