@@ -46,11 +46,46 @@ extern "C" {
 #define stricmp _stricmp
 #define strnicmp _strnicmp
 
-#include <float.h>
-#define isnan _isnan
+/* definition from winsock2.h */
+#ifndef _TIMEVAL_DEFINED
+#define _TIMEVAL_DEFINED
+typedef struct timeval {
+  long tv_sec;
+  long tv_usec;
+} timeval;
+#endif
 
-/* This is a SPARQL token define */
+struct timezone
+{
+  int tz_minuteswest; /* minutes West of Greenwich */
+  int tz_dsttime;     /* type of dst correction */
+};
+
+int rasqal_gettimeofday(struct timeval *tv, struct timezone *tz);
+#undef HAVE_GETTIMEOFDAY
+
+#include <float.h>
+#define isnan(n) _isnan(n)
+/* no round function available */
+#define round(x) floor(x+0.5)
+
+/* These are SPARQL token definitions */
+#ifdef OPTIONAL
 #undef OPTIONAL
+#endif
+
+#ifdef DELETE
+#undef DELETE
+#endif
+
+#ifdef IN
+#undef IN
+#endif
+
+#ifdef GROUP
+#undef GROUP
+#endif
+
 
 #define HAVE_C99_VSNPRINTF 1
 
@@ -71,10 +106,10 @@ extern "C" {
 #include <memory.h>
 
 /* Release version as a decimal */
-#define RASQAL_VERSION 925
+#define RASQAL_VERSION 927
 
 /* Version number of package */
-#define RASQAL_VERSION_STRING "0.9.25"
+#define RASQAL_VERSION_STRING "0.9.27"
 
 /* Major version number */
 #define RASQAL_VERSION_MAJOR 0
@@ -83,7 +118,7 @@ extern "C" {
 #define RASQAL_VERSION_MINOR 9
 
 /* Release version number */
-#define RASQAL_VERSION_RELEASE 25
+#define RASQAL_VERSION_RELEASE 27
 
 
 /* This is a SPARQL token define */
